@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option) 
+ * the Free Software Foundation; either version 3, or (at your option) 
  * any later version.
  *
  * This program is distributed in the hope that it will be useful, 
@@ -14,9 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License 
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
  */
@@ -35,6 +35,10 @@ extern struct layer *flayer;
 
 extern char *screenencodings;
 
+#ifdef DW_CHARS
+extern int cjkwidth;
+#endif
+
 static int  encmatch __P((char *, char *));
 # ifdef UTF8
 static int   recode_char __P((int, int, int));
@@ -510,6 +514,8 @@ int from, to;
   if (rmc.font == 0)	/* latin1 is the same in unicode */
     return mc;
   c = rmc.image | (rmc.font << 8);
+  if (from == UTF8)
+    c |= rmc.fontx << 16;
 #ifdef DW_CHARS
   if (rmc.mbcs)
     {
@@ -522,6 +528,8 @@ int from, to;
     c = recode_char_to_encoding(c, to);
   rmc.image = c & 255;
   rmc.font = c >> 8 & 255;
+  if (to == UTF8)
+    rmc.fontx = c >> 16 & 255;
   return &rmc;
 }
 
@@ -538,7 +546,7 @@ int from, to;
 
   if (from == to || (from != UTF8 && to != UTF8) || w == 0)
     return ml;
-  if (ml->font == null && encodings[from].deffont == 0)
+  if (ml->font == null && ml->fontx == null && encodings[from].deffont == 0)
     return ml;
   if (w > maxlen)
     {
@@ -552,7 +560,11 @@ int from, to;
 	    rml[i].font = malloc(w);
 	  else
 	    rml[i].font = realloc(rml[i].font, w);
-	  if (rml[i].image == 0 || rml[i].font == 0)
+	  if (rml[i].fontx == 0)
+	    rml[i].fontx = malloc(w);
+	  else
+	    rml[i].fontx = realloc(rml[i].fontx, w);
+	  if (rml[i].image == 0 || rml[i].font == 0 || rml[i].fontx == 0)
 	    {
 	      maxlen = 0;
 	      return ml;	/* sorry */
@@ -574,6 +586,12 @@ int from, to;
   for (i = 0; i < w; i++)
     debug1("%c", "0123456789abcdef"[(ml->font[i]     ) & 15]);
   debug("\n");
+  for (i = 0; i < w; i++)
+    debug1("%c", "0123456789abcdef"[(ml->fontx[i] >> 4) & 15]);
+  debug("\n");
+  for (i = 0; i < w; i++)
+    debug1("%c", "0123456789abcdef"[(ml->fontx[i]     ) & 15]);
+  debug("\n");
 
   rl = rml + last;
   rl->attr = ml->attr;
@@ -586,6 +604,8 @@ int from, to;
   for (i = 0; i < w; i++)
     {
       c = ml->image[i] | (ml->font[i] << 8);
+      if (from == UTF8)
+	c |= ml->fontx[i] << 16;
       if (from != UTF8 && c < 256)
 	c |= encodings[from].deffont << 8;
 #ifdef DW_CHARS
@@ -599,6 +619,8 @@ int from, to;
 	      i++;
 	      c2 = ml->image[i] | (ml->font[i] << 8);
 	      c = recode_char_dw_to_encoding(c, &c2, to);
+	      if (to == UTF8)
+	        rl->fontx[i - 1]  = c >> 16 & 255;
 	      rl->font[i - 1]  = c >> 8 & 255;
 	      rl->image[i - 1] = c      & 255;
 	      c = c2;
@@ -609,6 +631,8 @@ int from, to;
         c = recode_char_to_encoding(c, to);
       rl->image[i] = c & 255;
       rl->font[i] = c >> 8 & 255;
+      if (to == UTF8)
+        rl->fontx[i] = c >> 16 & 255;
     }
   last ^= 1;
   debug("recode_mline: to\n");
@@ -624,14 +648,20 @@ int from, to;
   for (i = 0; i < w; i++)
     debug1("%c", "0123456789abcdef"[(rl->font[i]     ) & 15]);
   debug("\n");
+  for (i = 0; i < w; i++)
+    debug1("%c", "0123456789abcdef"[(rl->fontx[i] >> 4) & 15]);
+  debug("\n");
+  for (i = 0; i < w; i++)
+    debug1("%c", "0123456789abcdef"[(rl->fontx[i]     ) & 15]);
+  debug("\n");
   return rl;
 }
 
 struct combchar {
-  unsigned short c1;
-  unsigned short c2;
-  unsigned short next;
-  unsigned short prev;
+  unsigned int c1;
+  unsigned int c2;
+  unsigned int next;
+  unsigned int prev;
 };
 struct combchar **combchars;
 
@@ -645,10 +675,20 @@ int c;
       AddUtf8(combchars[c - 0xd800]->c1);
       c = combchars[c - 0xd800]->c2;
     }
+  if (c >= 0x10000)
+    {
+      if (c >= 0x200000)
+	{
+	  AddChar((c & 0x3000000) >> 12 ^ 0xf8);
+	  c = (c & 0xffffff) ^ ((0xf0 ^ 0x80) << 18);
+	}
+      AddChar((c & 0x1fc0000) >> 18 ^ 0xf0);
+      c = (c & 0x3ffff) ^ ((0xe0 ^ 0x80) << 12);
+    }
   if (c >= 0x800)
     {
-      AddChar((c & 0xf000) >> 12 | 0xe0);
-      c = (c & 0x0fff) | 0x1000; 
+      AddChar((c & 0x7f000) >> 12 ^ 0xe0);
+      c = (c & 0x0fff) ^ ((0xc0 ^ 0x80) << 6);
     }
   if (c >= 0x80)
     {
@@ -679,10 +719,24 @@ char *p;
 int c;
 {
   int l = 1;
+  if (c >= 0x10000)
+    {
+      if (c >= 0x200000)
+	{
+	  if (p)
+	    *p++ = (c & 0x3000000) >> 12 ^ 0xf8;
+	  l++;
+	  c = (c & 0xffffff) ^ ((0xf0 ^ 0x80) << 18);
+	}
+      if (p)
+        *p++ = (c & 0x1fc0000) >> 18 ^ 0xf0;
+      l++;
+      c = (c & 0x3ffff) ^ ((0xe0 ^ 0x80) << 12);
+    }
   if (c >= 0x800)
     {
       if (p)
-	*p++ = (c & 0xf000) >> 12 | 0xe0; 
+	*p++ = (c & 0x7f000) >> 12 ^ 0xe0; 
       l++;
       c = (c & 0x0fff) | 0x1000; 
     }
@@ -754,8 +808,13 @@ int c, *utf8charp;
   *utf8charp = utf8char = (c & 0x80000000) ? c : 0;
   if (utf8char)
     return -1;
+#if 0
   if (c & 0xffff0000)
     c = UCS_REPL;	/* sorry, only know 16bit Unicode */
+#else
+  if (c & 0xff800000)
+    c = UCS_REPL;	/* sorry, only know 23bit Unicode */
+#endif
   if (c >= 0xd800 && (c <= 0xdfff || c == 0xfffe || c == 0xffff))
     c = UCS_REPL;	/* illegal code */
   return c;
@@ -799,23 +858,24 @@ int encoding;
 #else
       ml = &p->w_mlines[j];
 #endif
-      if (ml->font == null && encodings[p->w_encoding].deffont == 0)
+      if (ml->font == null && ml->fontx == 0 && encodings[p->w_encoding].deffont == 0)
 	continue;
       for (i = 0; i < p->w_width; i++)
 	{
 	  c = ml->image[i] | (ml->font[i] << 8);
+	  if (p->w_encoding == UTF8)
+	    c |= ml->fontx[i] << 16;
 	  if (p->w_encoding != UTF8 && c < 256)
 	    c |= encodings[p->w_encoding].deffont << 8;
 	  if (c < 256)
 	    continue;
 	  if (ml->font == null)
 	    {
-	      if ((ml->font = (unsigned char *)malloc(p->w_width + 1)) == 0)
+	      if ((ml->font = (unsigned char *)calloc(p->w_width + 1, 1)) == 0)
 		{
 		  ml->font = null;
 		  break;
 		}
-	      bzero(ml->font, p->w_width + 1);
 	    }
 #ifdef DW_CHARS
 	  if ((p->w_encoding != UTF8 && (c & 0x1f00) != 0 && (c & 0xe000) == 0) || (p->w_encoding == UTF8 && utf8_isdouble(c)))
@@ -826,8 +886,22 @@ int encoding;
 		{
 		  int c2;
 		  i++;
-		  c2 = ml->image[i] | (ml->font[i] << 8);
+		  c2 = ml->image[i] | (ml->font[i] << 8) | (ml->fontx[i] << 16);
 		  c = recode_char_dw_to_encoding(c, &c2, encoding);
+		  if (encoding == UTF8)
+		    {
+		      if (c > 0x10000 && ml->fontx == null)
+			{
+			  if ((ml->fontx = (unsigned char *)calloc(p->w_width + 1, 1)) == 0)
+			    {
+			      ml->fontx = null;
+			      break;
+			    }
+			}
+		      ml->fontx[i - 1]  = c >> 16 & 255;
+		    }
+		  else
+		    ml->fontx = null;
 		  ml->font[i - 1]  = c >> 8 & 255;
 		  ml->image[i - 1] = c      & 255;
 		  c = c2;
@@ -838,6 +912,20 @@ int encoding;
 	    c = recode_char_to_encoding(c, encoding);
 	  ml->image[i] = c & 255;
 	  ml->font[i] = c >> 8 & 255;
+	  if (encoding == UTF8)
+	    {
+	      if (c > 0x10000 && ml->fontx == null)
+		{
+		  if ((ml->fontx = (unsigned char *)calloc(p->w_width + 1, 1)) == 0)
+		    {
+		      ml->fontx = null;
+		      break;
+		    }
+		}
+	      ml->fontx[i]  = c >> 16 & 255;
+	    }
+	  else
+	    ml->fontx = null;
 	}
     }
   p->w_encoding = encoding;
@@ -845,22 +933,107 @@ int encoding;
 }
 
 #ifdef DW_CHARS
+struct interval {
+  int first;
+  int last;
+};
+
+/* auxiliary function for binary search in interval table */
+static int bisearch(int ucs, const struct interval *table, int max) {
+  int min = 0;
+  int mid;
+
+  if (ucs < table[0].first || ucs > table[max].last)
+    return 0;
+  while (max >= min) {
+    mid = (min + max) / 2;
+    if (ucs > table[mid].last)
+      min = mid + 1;
+    else if (ucs < table[mid].first)
+      max = mid - 1;
+    else
+      return 1;
+  }
+
+  return 0;
+}
+
 int
 utf8_isdouble(c)
 int c;
 {
-  return
-    (c >= 0x1100 &&
-     (c <= 0x115f ||                    /* Hangul Jamo init. consonants */
-      (c >= 0x2e80 && c <= 0xa4cf && (c & ~0x0011) != 0x300a &&
-       c != 0x303f) ||                  /* CJK ... Yi */
-      (c >= 0xac00 && c <= 0xd7a3) || /* Hangul Syllables */
-      (c >= 0xdf00 && c <= 0xdfff) || /* dw combining sequence */
-      (c >= 0xf900 && c <= 0xfaff) || /* CJK Compatibility Ideographs */
-      (c >= 0xfe30 && c <= 0xfe6f) || /* CJK Compatibility Forms */
-      (c >= 0xff00 && c <= 0xff5f) || /* Fullwidth Forms */
-      (c >= 0xffe0 && c <= 0xffe6) ||
-      (c >= 0x20000 && c <= 0x2ffff)));
+  /* sorted list of non-overlapping intervals of East Asian Ambiguous
+   * characters, generated by "uniset +WIDTH-A -cat=Me -cat=Mn -cat=Cf c" */
+  static const struct interval ambiguous[] = {
+    { 0x00A1, 0x00A1 }, { 0x00A4, 0x00A4 }, { 0x00A7, 0x00A8 },
+    { 0x00AA, 0x00AA }, { 0x00AE, 0x00AE }, { 0x00B0, 0x00B4 },
+    { 0x00B6, 0x00BA }, { 0x00BC, 0x00BF }, { 0x00C6, 0x00C6 },
+    { 0x00D0, 0x00D0 }, { 0x00D7, 0x00D8 }, { 0x00DE, 0x00E1 },
+    { 0x00E6, 0x00E6 }, { 0x00E8, 0x00EA }, { 0x00EC, 0x00ED },
+    { 0x00F0, 0x00F0 }, { 0x00F2, 0x00F3 }, { 0x00F7, 0x00FA },
+    { 0x00FC, 0x00FC }, { 0x00FE, 0x00FE }, { 0x0101, 0x0101 },
+    { 0x0111, 0x0111 }, { 0x0113, 0x0113 }, { 0x011B, 0x011B },
+    { 0x0126, 0x0127 }, { 0x012B, 0x012B }, { 0x0131, 0x0133 },
+    { 0x0138, 0x0138 }, { 0x013F, 0x0142 }, { 0x0144, 0x0144 },
+    { 0x0148, 0x014B }, { 0x014D, 0x014D }, { 0x0152, 0x0153 },
+    { 0x0166, 0x0167 }, { 0x016B, 0x016B }, { 0x01CE, 0x01CE },
+    { 0x01D0, 0x01D0 }, { 0x01D2, 0x01D2 }, { 0x01D4, 0x01D4 },
+    { 0x01D6, 0x01D6 }, { 0x01D8, 0x01D8 }, { 0x01DA, 0x01DA },
+    { 0x01DC, 0x01DC }, { 0x0251, 0x0251 }, { 0x0261, 0x0261 },
+    { 0x02C4, 0x02C4 }, { 0x02C7, 0x02C7 }, { 0x02C9, 0x02CB },
+    { 0x02CD, 0x02CD }, { 0x02D0, 0x02D0 }, { 0x02D8, 0x02DB },
+    { 0x02DD, 0x02DD }, { 0x02DF, 0x02DF }, { 0x0391, 0x03A1 },
+    { 0x03A3, 0x03A9 }, { 0x03B1, 0x03C1 }, { 0x03C3, 0x03C9 },
+    { 0x0401, 0x0401 }, { 0x0410, 0x044F }, { 0x0451, 0x0451 },
+    { 0x2010, 0x2010 }, { 0x2013, 0x2016 }, { 0x2018, 0x2019 },
+    { 0x201C, 0x201D }, { 0x2020, 0x2022 }, { 0x2024, 0x2027 },
+    { 0x2030, 0x2030 }, { 0x2032, 0x2033 }, { 0x2035, 0x2035 },
+    { 0x203B, 0x203B }, { 0x203E, 0x203E }, { 0x2074, 0x2074 },
+    { 0x207F, 0x207F }, { 0x2081, 0x2084 }, { 0x20AC, 0x20AC },
+    { 0x2103, 0x2103 }, { 0x2105, 0x2105 }, { 0x2109, 0x2109 },
+    { 0x2113, 0x2113 }, { 0x2116, 0x2116 }, { 0x2121, 0x2122 },
+    { 0x2126, 0x2126 }, { 0x212B, 0x212B }, { 0x2153, 0x2154 },
+    { 0x215B, 0x215E }, { 0x2160, 0x216B }, { 0x2170, 0x2179 },
+    { 0x2190, 0x2199 }, { 0x21B8, 0x21B9 }, { 0x21D2, 0x21D2 },
+    { 0x21D4, 0x21D4 }, { 0x21E7, 0x21E7 }, { 0x2200, 0x2200 },
+    { 0x2202, 0x2203 }, { 0x2207, 0x2208 }, { 0x220B, 0x220B },
+    { 0x220F, 0x220F }, { 0x2211, 0x2211 }, { 0x2215, 0x2215 },
+    { 0x221A, 0x221A }, { 0x221D, 0x2220 }, { 0x2223, 0x2223 },
+    { 0x2225, 0x2225 }, { 0x2227, 0x222C }, { 0x222E, 0x222E },
+    { 0x2234, 0x2237 }, { 0x223C, 0x223D }, { 0x2248, 0x2248 },
+    { 0x224C, 0x224C }, { 0x2252, 0x2252 }, { 0x2260, 0x2261 },
+    { 0x2264, 0x2267 }, { 0x226A, 0x226B }, { 0x226E, 0x226F },
+    { 0x2282, 0x2283 }, { 0x2286, 0x2287 }, { 0x2295, 0x2295 },
+    { 0x2299, 0x2299 }, { 0x22A5, 0x22A5 }, { 0x22BF, 0x22BF },
+    { 0x2312, 0x2312 }, { 0x2460, 0x24E9 }, { 0x24EB, 0x254B },
+    { 0x2550, 0x2573 }, { 0x2580, 0x258F }, { 0x2592, 0x2595 },
+    { 0x25A0, 0x25A1 }, { 0x25A3, 0x25A9 }, { 0x25B2, 0x25B3 },
+    { 0x25B6, 0x25B7 }, { 0x25BC, 0x25BD }, { 0x25C0, 0x25C1 },
+    { 0x25C6, 0x25C8 }, { 0x25CB, 0x25CB }, { 0x25CE, 0x25D1 },
+    { 0x25E2, 0x25E5 }, { 0x25EF, 0x25EF }, { 0x2605, 0x2606 },
+    { 0x2609, 0x2609 }, { 0x260E, 0x260F }, { 0x2614, 0x2615 },
+    { 0x261C, 0x261C }, { 0x261E, 0x261E }, { 0x2640, 0x2640 },
+    { 0x2642, 0x2642 }, { 0x2660, 0x2661 }, { 0x2663, 0x2665 },
+    { 0x2667, 0x266A }, { 0x266C, 0x266D }, { 0x266F, 0x266F },
+    { 0x273D, 0x273D }, { 0x2776, 0x277F }, { 0xE000, 0xF8FF },
+    { 0xFFFD, 0xFFFD }, { 0xF0000, 0xFFFFD }, { 0x100000, 0x10FFFD }
+  };
+
+  return ((c >= 0x1100 &&
+           (c <= 0x115f ||                    /* Hangul Jamo init. consonants */
+            c == 0x2329 || c == 0x232a ||
+            (c >= 0x2e80 && c <= 0xa4cf &&
+             c != 0x303f) ||                  /* CJK ... Yi */
+            (c >= 0xac00 && c <= 0xd7a3) || /* Hangul Syllables */
+            (c >= 0xf900 && c <= 0xfaff) || /* CJK Compatibility Ideographs */
+            (c >= 0xfe30 && c <= 0xfe6f) || /* CJK Compatibility Forms */
+            (c >= 0xff00 && c <= 0xff60) || /* Fullwidth Forms */
+            (c >= 0xffe0 && c <= 0xffe6) ||
+            (c >= 0x20000 && c <= 0x2fffd) ||
+            (c >= 0x30000 && c <= 0x3fffd))) ||
+          (cjkwidth &&
+           bisearch(c, ambiguous,
+	            sizeof(ambiguous) / sizeof(struct interval) - 1)));
 }
 #endif
 
@@ -869,62 +1042,58 @@ utf8_iscomb(c)
 int c;
 {
   /* taken from Markus Kuhn's wcwidth */
-  static struct {
-    unsigned short first;
-    unsigned short last;
-  } combining[] = {
-    { 0x0300, 0x034F }, { 0x0360, 0x036F }, { 0x0483, 0x0486 },
-    { 0x0488, 0x0489 }, { 0x0591, 0x05A1 }, { 0x05A3, 0x05B9 },
-    { 0x05BB, 0x05BD }, { 0x05BF, 0x05BF }, { 0x05C1, 0x05C2 },
-    { 0x05C4, 0x05C4 }, { 0x064B, 0x0655 }, { 0x0670, 0x0670 },
+  static const struct interval combining[] = {
+    { 0x0300, 0x036F }, { 0x0483, 0x0486 }, { 0x0488, 0x0489 },
+    { 0x0591, 0x05BD }, { 0x05BF, 0x05BF }, { 0x05C1, 0x05C2 },
+    { 0x05C4, 0x05C5 }, { 0x05C7, 0x05C7 }, { 0x0600, 0x0603 },
+    { 0x0610, 0x0615 }, { 0x064B, 0x065E }, { 0x0670, 0x0670 },
     { 0x06D6, 0x06E4 }, { 0x06E7, 0x06E8 }, { 0x06EA, 0x06ED },
     { 0x070F, 0x070F }, { 0x0711, 0x0711 }, { 0x0730, 0x074A },
-    { 0x07A6, 0x07B0 }, { 0x0901, 0x0902 }, { 0x093C, 0x093C },
-    { 0x0941, 0x0948 }, { 0x094D, 0x094D }, { 0x0951, 0x0954 },
-    { 0x0962, 0x0963 }, { 0x0981, 0x0981 }, { 0x09BC, 0x09BC },
-    { 0x09C1, 0x09C4 }, { 0x09CD, 0x09CD }, { 0x09E2, 0x09E3 },
-    { 0x0A02, 0x0A02 }, { 0x0A3C, 0x0A3C }, { 0x0A41, 0x0A42 },
-    { 0x0A47, 0x0A48 }, { 0x0A4B, 0x0A4D }, { 0x0A70, 0x0A71 },
-    { 0x0A81, 0x0A82 }, { 0x0ABC, 0x0ABC }, { 0x0AC1, 0x0AC5 },
-    { 0x0AC7, 0x0AC8 }, { 0x0ACD, 0x0ACD }, { 0x0B01, 0x0B01 },
-    { 0x0B3C, 0x0B3C }, { 0x0B3F, 0x0B3F }, { 0x0B41, 0x0B43 },
-    { 0x0B4D, 0x0B4D }, { 0x0B56, 0x0B56 }, { 0x0B82, 0x0B82 },
-    { 0x0BC0, 0x0BC0 }, { 0x0BCD, 0x0BCD }, { 0x0C3E, 0x0C40 },
-    { 0x0C46, 0x0C48 }, { 0x0C4A, 0x0C4D }, { 0x0C55, 0x0C56 },
+    { 0x07A6, 0x07B0 }, { 0x07EB, 0x07F3 }, { 0x0901, 0x0902 },
+    { 0x093C, 0x093C }, { 0x0941, 0x0948 }, { 0x094D, 0x094D },
+    { 0x0951, 0x0954 }, { 0x0962, 0x0963 }, { 0x0981, 0x0981 },
+    { 0x09BC, 0x09BC }, { 0x09C1, 0x09C4 }, { 0x09CD, 0x09CD },
+    { 0x09E2, 0x09E3 }, { 0x0A01, 0x0A02 }, { 0x0A3C, 0x0A3C },
+    { 0x0A41, 0x0A42 }, { 0x0A47, 0x0A48 }, { 0x0A4B, 0x0A4D },
+    { 0x0A70, 0x0A71 }, { 0x0A81, 0x0A82 }, { 0x0ABC, 0x0ABC },
+    { 0x0AC1, 0x0AC5 }, { 0x0AC7, 0x0AC8 }, { 0x0ACD, 0x0ACD },
+    { 0x0AE2, 0x0AE3 }, { 0x0B01, 0x0B01 }, { 0x0B3C, 0x0B3C },
+    { 0x0B3F, 0x0B3F }, { 0x0B41, 0x0B43 }, { 0x0B4D, 0x0B4D },
+    { 0x0B56, 0x0B56 }, { 0x0B82, 0x0B82 }, { 0x0BC0, 0x0BC0 },
+    { 0x0BCD, 0x0BCD }, { 0x0C3E, 0x0C40 }, { 0x0C46, 0x0C48 },
+    { 0x0C4A, 0x0C4D }, { 0x0C55, 0x0C56 }, { 0x0CBC, 0x0CBC },
     { 0x0CBF, 0x0CBF }, { 0x0CC6, 0x0CC6 }, { 0x0CCC, 0x0CCD },
-    { 0x0D41, 0x0D43 }, { 0x0D4D, 0x0D4D }, { 0x0DCA, 0x0DCA },
-    { 0x0DD2, 0x0DD4 }, { 0x0DD6, 0x0DD6 }, { 0x0E31, 0x0E31 },
-    { 0x0E34, 0x0E3A }, { 0x0E47, 0x0E4E }, { 0x0EB1, 0x0EB1 },
-    { 0x0EB4, 0x0EB9 }, { 0x0EBB, 0x0EBC }, { 0x0EC8, 0x0ECD },
-    { 0x0F18, 0x0F19 }, { 0x0F35, 0x0F35 }, { 0x0F37, 0x0F37 },
-    { 0x0F39, 0x0F39 }, { 0x0F71, 0x0F7E }, { 0x0F80, 0x0F84 },
-    { 0x0F86, 0x0F87 }, { 0x0F90, 0x0F97 }, { 0x0F99, 0x0FBC },
-    { 0x0FC6, 0x0FC6 }, { 0x102D, 0x1030 }, { 0x1032, 0x1032 },
-    { 0x1036, 0x1037 }, { 0x1039, 0x1039 }, { 0x1058, 0x1059 },
-    { 0x1160, 0x11FF }, { 0x1712, 0x1714 }, { 0x1732, 0x1734 },
-    { 0x1752, 0x1753 }, { 0x1772, 0x1773 }, { 0x17B7, 0x17BD },
-    { 0x17C6, 0x17C6 }, { 0x17C9, 0x17D3 }, { 0x180B, 0x180E },
-    { 0x18A9, 0x18A9 }, { 0x200B, 0x200F }, { 0x202A, 0x202E },
-    { 0x2060, 0x2063 }, { 0x206A, 0x206F }, { 0x20D0, 0x20EA },
-    { 0x302A, 0x302F }, { 0x3099, 0x309A }, { 0xFB1E, 0xFB1E },
-    { 0xFE00, 0xFE0F }, { 0xFE20, 0xFE23 }, { 0xFEFF, 0xFEFF },
-    { 0xFFF9, 0xFFFB }
+    { 0x0CE2, 0x0CE3 }, { 0x0D41, 0x0D43 }, { 0x0D4D, 0x0D4D },
+    { 0x0DCA, 0x0DCA }, { 0x0DD2, 0x0DD4 }, { 0x0DD6, 0x0DD6 },
+    { 0x0E31, 0x0E31 }, { 0x0E34, 0x0E3A }, { 0x0E47, 0x0E4E },
+    { 0x0EB1, 0x0EB1 }, { 0x0EB4, 0x0EB9 }, { 0x0EBB, 0x0EBC },
+    { 0x0EC8, 0x0ECD }, { 0x0F18, 0x0F19 }, { 0x0F35, 0x0F35 },
+    { 0x0F37, 0x0F37 }, { 0x0F39, 0x0F39 }, { 0x0F71, 0x0F7E },
+    { 0x0F80, 0x0F84 }, { 0x0F86, 0x0F87 }, { 0x0F90, 0x0F97 },
+    { 0x0F99, 0x0FBC }, { 0x0FC6, 0x0FC6 }, { 0x102D, 0x1030 },
+    { 0x1032, 0x1032 }, { 0x1036, 0x1037 }, { 0x1039, 0x1039 },
+    { 0x1058, 0x1059 }, { 0x1160, 0x11FF }, { 0x135F, 0x135F },
+    { 0x1712, 0x1714 }, { 0x1732, 0x1734 }, { 0x1752, 0x1753 },
+    { 0x1772, 0x1773 }, { 0x17B4, 0x17B5 }, { 0x17B7, 0x17BD },
+    { 0x17C6, 0x17C6 }, { 0x17C9, 0x17D3 }, { 0x17DD, 0x17DD },
+    { 0x180B, 0x180D }, { 0x18A9, 0x18A9 }, { 0x1920, 0x1922 },
+    { 0x1927, 0x1928 }, { 0x1932, 0x1932 }, { 0x1939, 0x193B },
+    { 0x1A17, 0x1A18 }, { 0x1B00, 0x1B03 }, { 0x1B34, 0x1B34 },
+    { 0x1B36, 0x1B3A }, { 0x1B3C, 0x1B3C }, { 0x1B42, 0x1B42 },
+    { 0x1B6B, 0x1B73 }, { 0x1DC0, 0x1DCA }, { 0x1DFE, 0x1DFF },
+    { 0x200B, 0x200F }, { 0x202A, 0x202E }, { 0x2060, 0x2063 },
+    { 0x206A, 0x206F }, { 0x20D0, 0x20EF }, { 0x302A, 0x302F },
+    { 0x3099, 0x309A }, { 0xA806, 0xA806 }, { 0xA80B, 0xA80B },
+    { 0xA825, 0xA826 }, { 0xFB1E, 0xFB1E }, { 0xFE00, 0xFE0F },
+    { 0xFE20, 0xFE23 }, { 0xFEFF, 0xFEFF }, { 0xFFF9, 0xFFFB },
+    { 0x10A01, 0x10A03 }, { 0x10A05, 0x10A06 }, { 0x10A0C, 0x10A0F },
+    { 0x10A38, 0x10A3A }, { 0x10A3F, 0x10A3F }, { 0x1D167, 0x1D169 },
+    { 0x1D173, 0x1D182 }, { 0x1D185, 0x1D18B }, { 0x1D1AA, 0x1D1AD },
+    { 0x1D242, 0x1D244 }, { 0xE0001, 0xE0001 }, { 0xE0020, 0xE007F },
+    { 0xE0100, 0xE01EF }
   };
-  int mid, min = 0, max = sizeof(combining)/sizeof(*combining) - 1;
 
-  if (c < 0x0300 || c > 0xfffb)
-    return 0;
-  while (max >= min)
-    {
-      mid = (min + max) / 2;
-      if (c > combining[mid].last)
-	min = mid + 1;
-      else if (c < combining[mid].first)
-	max = mid - 1;
-      else
-	return 1;
-    }
-  return 0;
+  return bisearch(c, combining, sizeof(combining) / sizeof(struct interval) - 1);
 }
 
 static void
@@ -955,14 +1124,13 @@ struct mchar *mc;
   int root, i, c1;
   int isdouble;
 
-  c1 = mc->image | (mc->font << 8);
+  c1 = mc->image | (mc->font << 8) | mc->fontx << 16;
   isdouble = c1 >= 0x1100 && utf8_isdouble(c1);
   if (!combchars)
     {
-      combchars = (struct combchar **)malloc(sizeof(struct combchar *) * 0x802);
+      combchars = (struct combchar **)calloc(0x802, sizeof(struct combchar *));
       if (!combchars)
 	return;
-      bzero((char *)combchars, sizeof(struct combchar *) * 0x802);
       combchars[0x800] = (struct combchar *)malloc(sizeof(struct combchar));
       combchars[0x801] = (struct combchar *)malloc(sizeof(struct combchar));
       if (!combchars[0x800] || !combchars[0x801])
@@ -997,7 +1165,7 @@ struct mchar *mc;
       if (c1 >= 0xd800 && c1 < 0xe000)
         comb_tofront(root, c1 - 0xd800);
       i = combchars[root]->prev;
-      if (c1 == i + 0xd800) 
+      if (c1 == i + 0xd800)
 	{
 	  /* completely full, can't recycle */
 	  debug("utf8_handle_comp: completely full!\n");
@@ -1019,6 +1187,7 @@ struct mchar *mc;
   combchars[i]->c2 = c;
   mc->image = i & 0xff;
   mc->font  = (i >> 8) + 0xd8;
+  mc->fontx = 0;
   debug3("combinig char %x %x -> %x\n", c1, c, i + 0xd800);
   comb_tofront(root, i);
 }
@@ -1137,6 +1306,15 @@ struct win *p;
     p->w_c1 = 0;
 }
 
+/* decoded char: 32-bit <fontx><font><c2><c>
+ * fontx: non-bmp utf8
+ * c2: multi-byte character
+ * font is always zero for utf8
+ * returns: -1 need more bytes
+ *          -2 decode error
+ */
+
+
 int
 DecodeChar(c, encoding, statep)
 int c;
@@ -1148,7 +1326,12 @@ int *statep;
   debug2("Decoding char %02x for encoding %d\n", c, encoding);
 #ifdef UTF8
   if (encoding == UTF8)
-    return FromUtf8(c, statep);
+    {
+      c = FromUtf8(c, statep);
+      if (c >= 0x10000)
+	c = (c & 0x7f0000) << 8 | (c & 0xffff);
+      return c;
+    }
 #endif
   if (encoding == SJIS)
     {
@@ -1159,6 +1342,8 @@ int *statep;
 	      *statep = c;
 	      return -1;
 	    }
+	  if (c < 0x80)
+	    return c;
 	  return c | (KANA << 16);
 	}
       t = c;
@@ -1166,11 +1351,16 @@ int *statep;
       *statep = 0;
       if (0x40 <= t && t <= 0xfc && t != 0x7f)
 	{
-	  if (c <= 0x9f) c = (c - 0x81) * 2 + 0x21; 
-	  else c = (c - 0xc1) * 2 + 0x21; 
-	  if (t <= 0x7e) t -= 0x1f;
-	  else if (t <= 0x9e) t -= 0x20;
-	  else t -= 0x7e, c++;
+	  if (c <= 0x9f)
+	    c = (c - 0x81) * 2 + 0x21; 
+	  else
+	    c = (c - 0xc1) * 2 + 0x21; 
+	  if (t <= 0x7e)
+	    t -= 0x1f;
+	  else if (t <= 0x9e)
+	    t -= 0x20;
+	  else
+	     t -= 0x7e, c++;
 	  return (c << 8) | t | (KANJI << 16);
 	}
       return t;
@@ -1255,7 +1445,7 @@ int *fontp;
 	}
       return 3;
     }
-  f = c >> 16;
+  f = (c >> 16) & 0xff;
 
 #ifdef UTF8
   if (encoding == UTF8)
@@ -1278,8 +1468,10 @@ int *fontp;
         }
       return ToUtf8(bp, c);
     }
-  if ((c & 0xff00) && f == 0)	/* is_utf8? */
+  if (f == 0 && (c & 0x7f00ff00) != 0)	/* is_utf8? */
     {
+      if (c >= 0x10000)
+	c = (c & 0x7f0000) >> 8 | (c & 0xffff);
 # ifdef DW_CHARS
       if (utf8_isdouble(c))
 	{
