@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,9 +19,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
  */
@@ -52,7 +57,7 @@ register const char *str;
   register char *cp;
 
   if ((cp = malloc(strlen(str) + 1)) == NULL)
-    Panic(0, strnomem);
+    Panic(0, "%s", strnomem);
   else
     strcpy(cp, str);
   return cp;
@@ -66,7 +71,7 @@ int n;
   register char *cp;
 
   if ((cp = malloc(n + 1)) == NULL)
-    Panic(0, strnomem);
+    Panic(0, "%s", strnomem);
   else
     {
       bcopy((char *)str, cp, n);
@@ -120,9 +125,10 @@ int y;
 }
 
 void
-leftline(str, y)
+leftline(str, y, rend)
 char *str;
 int y;
+struct mchar *rend;
 {
   int l, n;
   struct mchar mchar_dol;
@@ -134,7 +140,7 @@ int y;
   l = n = strlen(str);
   if (n > flayer->l_width - 1)
     n = flayer->l_width - 1;
-  LPutStr(flayer, str, n, &mchar_blank, 0, y);
+  LPutStr(flayer, str, n, rend ? rend : &mchar_blank, 0, y);
   if (n != l)
     LPutChar(flayer, &mchar_dol, n, y);
 }
@@ -574,80 +580,6 @@ int msec;
 }
 
 
-#if 0
-struct win **
-GlobWindows(pattern)
-char *pattern;
-{
-  static struct win *wv[MAXWIN + 1];
-  extern struct win *windows;
-  struct win *w;
-  struct win **av;
-  char *p;
-  int t, l, ll, i;
-
-  av = wv;
-
-  if (!pattern)
-    return NULL;
-  debug1("GlobWindows pattern '%s'\n", pattern);
-  t = 0;
-  p = pattern;
-  if (*p == '*')
-    {
-      t++;
-      p++;
-    }
-  l = strlen(p);
-  if (l && p[l - 1] == '*')
-    {
-      l--;
-      p[l] = '\0';
-      t += 2;
-    }
-
-  for (w = windows; w; w = w->w_next)
-    {
-      switch (t)
-	{
-	case 0:         /* exact match */
-	  if (!strcmp(p, w->w_title))
-	    *av++ = w;
-	  break;
-	case 1:         /* suffix match */
-	  ll = strlen(w->w_title);
-	  if (l < ll && !strncmp(p, w->w_title + ll - l, l))
-	    *av++ = w;
-	  break;
-	case 2:         /* prefix match */
-	  if (!strncmp(p, w->w_title, l))
-	    *av++ = w;
-	  break;
-	default:        /* 3: infix match */
-	  ll = strlen(w->w_title);
-	  for (i = ll - l; i >= 0; i--)
-	    {
-	      if (!strncmp(p, w->w_title + i, l))
-		{
-		  *av++ = w;
-		  break;
-		}
-	    }
-	}
-    }
-  *av = NULL;
-#ifdef DEBUG
-  {
-    struct win **pp = wv;
-
-    while (--pp >= wv)
-      debug1("GlobWindows: '%s'\n", (*pp)->w_title);
-  }
-#endif
-  return (av != wv) ? wv : NULL;
-}
-#endif
-
 /*
  * This uses either setenv() or putenv(). If it is putenv() we cannot dare
  * to free the buffer after putenv(), unless it it the one found in putenv.c
@@ -677,7 +609,7 @@ char *value;
    * the string space, we can free our buf now.
    */
   free(buf);
-# else /* NEEDSETENV */
+# else /* NEEDPUTENV */
   /*
    * For all sysv-ish systems that link a standard putenv()
    * the string-space buf is added to the environment and must not
@@ -685,13 +617,13 @@ char *value;
    * We are sorry to say that memory is lost here, when setting
    * the same variable again and again.
    */
-# endif /* NEEDSETENV */
+# endif /* NEEDPUTENV */
 #else /* USESETENV */
-# if defined(linux) || defined(__convex__) || (BSD >= 199103)
+# if HAVE_SETENV_3
   setenv(var, value, 1);
 # else
   setenv(var, value);
-# endif /* linux || convex || BSD >= 199103 */
+# endif /* HAVE_SETENV_3 */
 #endif /* USESETENV */
 }
 
@@ -720,39 +652,6 @@ int (*outc) __P((int));
   return 0;
 }
 
-# ifdef linux
-
-/* stupid stupid linux ncurses! It won't to padding with
- * zeros but sleeps instead. This breaks CalcCost, of course.
- * Also, the ncurses wait functions use a global variable
- * to store the current outc function. Oh well...
- */
-
-int (*save_outc) __P((int));
-
-#  undef tputs
-
-void
-xtputs(str, affcnt, outc)
-char *str;
-int affcnt;
-int (*outc) __P((int));
-{
-  extern int tputs __P((const char *, int, int (*)(int)));
-  save_outc = outc;
-  tputs(str, affcnt, outc);
-}
-
-int
-_nc_timed_wait(mode, ms, tlp)
-int mode, ms, *tlp;
-{
-  _delay(ms * 10, save_outc);
-  return 0;
-}
-
-# endif /* linux */
-
 #endif /* TERMINFO */
 
 
