@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +19,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
- * $Id: screen.h,v 1.12 1994/05/31 12:32:54 mlschroe Exp $ FAU
+ * $Id$ GNU
  */
 
 #include "os.h"
@@ -86,7 +91,7 @@
 
 #define Ctrl(c) ((c)&037)
 
-#define MAXSTR		256
+#define MAXSTR		768
 #define MAXARGS 	64
 #define MSGWAIT 	5
 #define MSGMINWAIT 	1
@@ -141,6 +146,7 @@ struct mode
 
 /* #include "logfile.h" */	/* (requires stat.h) struct logfile */
 #include "image.h"
+#include "canvas.h"
 #include "display.h"
 #include "window.h"
 
@@ -167,12 +173,20 @@ struct mode
 #define MSG_WINCH	6
 #define MSG_HANGUP	7
 #define MSG_COMMAND	8
+#define MSG_QUERY       9
 
 /*
  * versions of struct msg:
  * 0:	screen version 3.6.6 (version count introduced)
+ * 1:	screen version 4.1.0devel	(revisions e3fc19a upto 8147d08)
+ * 					 A few revisions after 8147d08 incorrectly
+ * 					 carried version 1, but should have carried 2.
+ * 2:	screen version 4.1.0devel	(revisions 8b46d8a upto YYYYYYY)
+ * 3:	screen version 4.2.0		(was incorrectly originally. Patched here)
+ * 4:	screen version 4.2.1		(bumped once again due to changed terminal and login length)
  */
-#define MSG_VERSION	0	
+#define MSG_VERSION	4
+
 #define MSG_REVISION	(('m'<<24) | ('s'<<16) | ('g'<<8) | MSG_VERSION)
 struct msg
 {
@@ -190,35 +204,38 @@ struct msg
 	  int nargs;
 	  char line[MAXPATHLEN];
 	  char dir[MAXPATHLEN];
-	  char screenterm[20];	/* is screen really "screen" ? */
+	  char screenterm[MAXTERMLEN];	/* is screen really "screen" ? */
 	}
       create;
       struct
 	{
-	  char auser[20 + 1];	/* username */
+	  char auser[MAXLOGINLEN + 1];	/* username */
 	  int apid;		/* pid of frontend */
 	  int adaptflag;	/* adapt window size? */
 	  int lines, columns;	/* display size */
 	  char preselect[20];
 	  int esc;		/* his new escape character unless -1 */
 	  int meta_esc;		/* his new meta esc character unless -1 */
-	  char envterm[20 + 1];	/* terminal type */
+	  char envterm[MAXTERMLEN + 1];	/* terminal type */
 	  int encoding;		/* encoding of display */
+	  int detachfirst;      /* whether to detach remote sessions first */
 	}
       attach;
       struct 
 	{
-	  char duser[20 + 1];	/* username */
+	  char duser[MAXLOGINLEN + 1];	/* username */
 	  int dpid;		/* pid of frontend */
 	}
       detach;
       struct 
 	{
-	  char auser[20 + 1];	/* username */
+	  char auser[MAXLOGINLEN + 1];	/* username */
 	  int nargs;
 	  char cmd[MAXPATHLEN];	/* command */
 	  int apid;		/* pid of frontend */
 	  char preselect[20];
+	  char writeback[MAXPATHLEN];  /* The socket to write the result.
+					  Only used for MSG_QUERY */
 	}
       command;
       char message[MAXPATHLEN * 2];
@@ -241,7 +258,7 @@ struct msg
 #define VBELLWAIT	1 /* No. of seconds a vbell will be displayed */
 
 #define BELL_ON		0 /* No bell has occurred in the window */
-#define BELL_FOUND 	1 /* A bell has occurred, but user not yet notified */
+#define BELL_FOUND	1 /* A bell has occurred, but user not yet notified */
 #define BELL_DONE	2 /* A bell has occured, user has been notified */
 
 #define BELL_VISUAL	3 /* A bell has occured in fore win, notify him visually */
@@ -256,8 +273,10 @@ struct msg
 #define DUMP_EXCHANGE	2
 #define DUMP_SCROLLBACK 3
 
-#define SILENCE_OFF	0
-#define SILENCE_ON	1
+#define SILENCE_OFF	0 /* Not checking for silence */
+#define SILENCE_ON	1 /* Window being monitored for silence */
+#define SILENCE_FOUND   2 /* Window is silent */
+#define SILENCE_DONE    3 /* Window is silent and user is notified */
 
 extern char strnomem[];
 
@@ -287,3 +306,10 @@ struct baud_values
   int bps;	/* bits per seconds */
   int sym;	/* symbol defined in ttydev.h */
 };
+
+/*
+ * windowlist orders
+ */
+#define WLIST_NUM 0
+#define WLIST_MRU 1
+#define WLIST_NESTED 2
