@@ -87,6 +87,7 @@ static int  DoAutolf __P((char *, int *, int));
 static void ZombieProcess __P((char **, int *));
 static void win_readev_fn __P((struct event *, char *));
 static void win_writeev_fn __P((struct event *, char *));
+static void win_resurrect_zombie_fn __P((struct event *, char *));
 static int  muchpending __P((struct win *, struct event *));
 #ifdef COPY_PASTE
 static void paste_slowev_fn __P((struct event *, char *));
@@ -139,7 +140,8 @@ struct NewWindow nwin_undef   =
   -1, 		/* bce */
   -1, 		/* encoding */
   (char *)0,	/* hstatus */
-  (char *)0	/* charset */
+  (char *)0,	/* charset */
+  0		/* poll_zombie_timeout */
 };
 
 struct NewWindow nwin_default = 
@@ -198,6 +200,7 @@ struct NewWindow *def, *new, *res;
   COMPOSE(encoding);
   COMPOSE(hstatus);
   COMPOSE(charset);
+  COMPOSE(poll_zombie_timeout);
 #undef COMPOSE
 }
 
@@ -611,6 +614,13 @@ struct NewWindow *newwin;
   n = pp - wtab;
   debug1("Makewin creating %d\n", n);
 
+#ifdef BUILTIN_TELNET
+	if(!strcmp(nwin.args[0], "//telnet")) {
+		type = W_TYPE_TELNET;
+		TtyName = "telnet";
+	}
+  else
+#endif
   if ((f = OpenDevice(nwin.args, nwin.lflag, &type, &TtyName)) < 0)
     return -1;
   if (type == W_TYPE_GROUP)
@@ -772,7 +782,7 @@ struct NewWindow *newwin;
 #ifdef BUILTIN_TELNET
   if (type == W_TYPE_TELNET)
     {
-      if (TelConnect(p))
+      if (TelOpenAndConnect(p))
 	{
 	  FreeWindow(p);
 	  return -1;
@@ -835,6 +845,14 @@ struct NewWindow *newwin;
       DoStartLog(p, buf, sizeof(buf));
     }
 
+   /* Is this all where I have to init window poll timeout? */
+	if (nwin.poll_zombie_timeout)
+		p->w_poll_zombie_timeout = nwin.poll_zombie_timeout;
+
+	p->w_zombieev.type = EV_TIMEOUT;
+	p->w_zombieev.data = (char *)p;
+	p->w_zombieev.handler = win_resurrect_zombie_fn;
+
   p->w_readev.fd = p->w_writeev.fd = p->w_ptyfd;
   p->w_readev.type = EV_READ;
   p->w_writeev.type = EV_WRITE;
@@ -884,6 +902,13 @@ struct win *p;
   int lflag, f;
 
   lflag = nwin_default.lflag;
+#ifdef BUILTIN_TELNET
+	if(!strcmp(p->w_cmdargs[0], "//telnet")) {
+		p->w_type = W_TYPE_TELNET;
+		TtyName = "telnet";
+	}
+	else
+#endif
   if ((f = OpenDevice(p->w_cmdargs, lflag, &p->w_type, &TtyName)) < 0)
     return -1;
 
@@ -917,7 +942,7 @@ struct win *p;
 #ifdef BUILTIN_TELNET
   if (p->w_type == W_TYPE_TELNET)
     {
-      if (TelConnect(p))
+      if (TelOpenAndConnect(p))
         return -1;
     }
   else
@@ -1050,6 +1075,7 @@ struct win *wp;
   evdeq(&wp->w_readev);		/* just in case */
   evdeq(&wp->w_writeev);	/* just in case */
   evdeq(&wp->w_silenceev);
+  evdeq(&wp->w_zombieev);
   evdeq(&wp->w_destroyev);
 #ifdef COPY_PASTE
   FreePaster(&wp->w_paster);
@@ -1076,16 +1102,6 @@ char **namep;
       *namep = "telnet";
       return 0;
     }
-#ifdef BUILTIN_TELNET
-  if (strcmp(arg, "//telnet") == 0)
-    {
-      f = TelOpen(args + 1);
-      lflag = 0;
-      *typep = W_TYPE_TELNET;
-      *namep = "telnet";
-    }
-  else
-#endif
   if (strncmp(arg, "//", 2) == 0)
     {
       Msg(0, "Invalid argument '%s'", arg);
@@ -1941,6 +1957,21 @@ char *data;
   return;
 }
 
+static void
+win_resurrect_zombie_fn(ev, data)
+struct event *ev;
+char *data;
+{
+	struct win *p = (struct win *)data;
+	debug2("Try to resurrecting Zombie event: %d [%s]\n",
+	p->w_number, p->w_title);
+	/* Already reconnected? */
+	if (p->w_deadpid != p->w_pid)
+		return;
+	debug1("Resurrecting Zombie: %d\n", p->w_number);
+	WriteString(p, "\r\n", 2);
+	RemakeWindow(p);
+}
 
 static void
 win_writeev_fn(ev, data)
@@ -2277,49 +2308,43 @@ struct display *d;
 #endif
 
 int
-WindowChangeNumber(struct win *win, int n)
+WindowChangeNumber(int old, int dest)
 {
-  struct win *p;
-  int old = win->w_number;
+  struct win *p, *win_old;
 
-  if (n < 0 || n >= maxwin)
+  if (dest < 0 || dest >= maxwin)
     {
       Msg(0, "Given window position is invalid.");
       return 0;
     }
 
-  p = wtab[n];
-  wtab[n] = win;
-  win->w_number = n;
+  win_old = wtab[old];
+  p = wtab[dest];
+  wtab[dest] = win_old;
+  win_old->w_number = dest;
   wtab[old] = p;
   if (p)
     p->w_number = old;
 #ifdef MULTIUSER
   /* exchange the acls for these windows. */
-  AclWinSwap(old, n);
+  AclWinSwap(old, dest);
 #endif
 #ifdef UTMPOK
   /* exchange the utmp-slots for these windows */
-  if ((win->w_slot != (slot_t) -1) && (win->w_slot != (slot_t) 0))
+  if ((win_old->w_slot != (slot_t) -1) && (win_old->w_slot != (slot_t) 0))
     {
-      RemoveUtmp(win);
-      SetUtmp(win);
+      RemoveUtmp(win_old);
+      SetUtmp(win_old);
     }
   if (p && (p->w_slot != (slot_t) -1) && (p->w_slot != (slot_t) 0))
     {
-      /* XXX: first display wins? */
-#if 0
-      /* Does this make more sense? */
-      display = p->w_lastdisp ? p->w_lastdisp : p->w_layer.l_cvlist ? p->w_layer.l_cvlist->c_display : 0;
-#else
-      display = win->w_layer.l_cvlist ? win->w_layer.l_cvlist->c_display : 0;
-#endif
+      display = win_old->w_layer.l_cvlist ? win_old->w_layer.l_cvlist->c_display : 0;
       RemoveUtmp(p);
       SetUtmp(p);
     }
 #endif
 
-  WindowChanged(win, 'n');
+  WindowChanged(win_old, 'n');
   WindowChanged((struct win *)0, 'w');
   WindowChanged((struct win *)0, 'W');
   WindowChanged((struct win *)0, 0);
